@@ -24,7 +24,7 @@
   * @author Samuel Mimram
   *)
 
-(* $Id: mad.ml 5568 2008-05-18 00:44:33Z metamorph68 $ *)
+(* $Id$ *)
 
 type mad_file
 
@@ -45,95 +45,58 @@ external openfile : string -> mad_file = "ocaml_mad_openfile"
 
 external openstream : (int -> (string * int)) -> mad_file = "ocaml_mad_openstream"
 
+external skip_id3tags : (int -> (string * int)) -> (int -> int) -> (unit -> int) -> unit = "ocaml_mad_skip_id3tag"
+
+let skip_id3tags ~read ~seek ~tell =
+  skip_id3tags read seek tell
+
 external close : mad_file -> unit = "ocaml_mad_close"
 
 external get_current_position : mad_file -> int = "ocaml_mad_get_current_position"
 
+external get_current_time : mad_file -> int -> int = "ocaml_mad_time"
+
+type time_unit = Hours | Minutes | Seconds | Deciseconds | Centiseconds | Milliseconds
+
+let int_of_time_unit =
+  function 
+    | Hours -> -2
+    | Minutes -> -1
+    | Seconds -> 0
+    | Deciseconds -> 10
+    | Centiseconds -> 100
+    | Milliseconds -> 1000
+
+let get_current_time dec u = 
+  get_current_time dec (int_of_time_unit u)
+
+external skip_frame : mad_file -> unit = "ocaml_mad_skip_frame"
+
 external decode_frame : mad_file -> string = "ocaml_mad_decode_frame"
 
 external decode_frame_float : mad_file -> float array array = "ocaml_mad_decode_frame_float"
 
 external get_output_format : mad_file -> int * int * int = "ocaml_mad_get_synth_pcm_format"
 
-let wav_output_channels = 2
-let wav_output_sample_size = 16
-let wav_output_big_endian = false
-let wav_output_signed = true
-
-let get_samplefreq f =
-  let bits_in i pos length =
-    ((int_of_char i) lsr pos) mod (1 lsl length)
-  in
-  let samplefreq_array =
-    [|
-      [| 11025; 12000; 8000; 0 |];
-      [| |];
-      [| 22050; 24000; 16000; 0 |];
-      [| 44100; 48000; 32000; 0 |];
-    |]
-  in
-  let read n =
-    let ans = String.create n in
-      assert (n = Unix.read f ans 0 n) ;
-      ans
-  in
-  let read_byte () =
-    int_of_char (read 1).[0]
-  in
-  let read_size () =
-    let buf = read 4 in
-    let b0 = int_of_char buf.[0] in
-    let b1 = int_of_char buf.[1] in
-    let b2 = int_of_char buf.[2] in
-    let b3 = int_of_char buf.[3] in
-      (* TODO* : lsl 7 -> overlapping ?? *)
-      (((((b0 lsl 7) lor b1) lsl 7) lor b2) lsl 7) lor b3
-  in
-    if read 3 = "ID3" then
-      (
-        ignore (Unix.lseek f 3 Unix.SEEK_CUR);
-        ignore (Unix.lseek f (read_size ()) Unix.SEEK_CUR);
-      )
-    else
-      ignore (Unix.lseek f 0 Unix.SEEK_SET);
-    (* Try to seek to the next frame. *)
-    while read_byte () <> 0xff do () done;
-    let buf = read 3 in
-    let version = bits_in buf.[0] 3 2 in
-    let samplefreq_i = bits_in buf.[1] 2 2 in
-      samplefreq_array.(version).(samplefreq_i)
-
-let samplefreq fname =
-  let f = Unix.openfile fname [Unix.O_RDONLY] 0o400 in
-  try
-    let freq = get_samplefreq f in
-      Unix.close f ;
-      freq
-  with
-    | e -> Unix.close f ; raise e
-
 let duration file =
-  let bytes mf =
-    let bytes = ref 0 in
-      try
-        while true do
-          bytes := !bytes + String.length (decode_frame mf)
-        done ;
-        assert false
-      with _ -> !bytes
+  let mf = openfile file in
+  let close () = 
+    try
+      close mf
+    with _ -> ()
   in
   try
-    let mf = openfile file in
+    begin 
+     try
+       while true do
+         skip_frame mf
+       done
+     with _ -> ()
+    end;
     let ret = 
-      try
-        let samplefreq = samplefreq file in
-          (float (8 * bytes mf)) /.
-          (float (wav_output_channels*wav_output_sample_size*samplefreq))
-      with _ -> 0.
+      (float (get_current_time mf Centiseconds)) /. 100. 
     in
-    try 
-      close mf ;
-      ret
-    with _ -> ret
+    close ();
+    ret
   with
-    | _ -> 0.
+    | _ -> close (); 0.
